
/*
 * Copyright (c) 2020 Circuit Dojo LLC
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#ifndef ZEPHYR_DRIVERS_RTC_PCF85263A_PCF85263A_H_
#define ZEPHYR_DRIVERS_RTC_PCF85263A_PCF85263A_H_

// CONTROL REGISTERS
#define PCF85263A_OSCILLATOR_REG 0x25
#define PCF85263A_OSCILLATOR_CLKIV BIT(7)
#define PCF85263A_OSCILLATOR_OFFM BIT(6)
#define PCF85263A_OSCILLATOR_12_24 BIT(5)
#define PCF85263A_OSCILLATOR_LOWJ BIT(4)
// enum {

// }PCF85263A_OSCILLATOR_OSCD;

// enum {

// }PCF85263A_OSCILLATOR_CL;

#define PCF85263A_BATTERY_SWITCH_REG 0x26
#define PCF85263A_BATTERY_SWITCH_BSOFF BIT(7)
#define PCF85263A_BATTERY_SWITCH_BSRR BIT(6)
#define PCF85263A_BATTERY_SWITCH_BSTH BIT(5)

// enum {

// }PCF85263A_OSCILLATOR_BSM;

#define PCF85263A_PIN_IO_REG 0x27
#define PCF85263A_PIN_IO_CLKPM BIT(7)
#define PCF85263A_PIN_IO_TSPULL BIT(6)
#define PCF85263A_PIN_IO_TSL BIT(5)
#define PCF85263A_PIN_IO_TSIM BIT(4)
// enum {

// }PCF85263A_PIN_IO_TSPM;

// enum {

// }PCF85263A_PIN_INTAPM;

#define PCF85263A_FUNCTION_REG 0x28
#define PCF85263A_FUNCTION_100TH BIT(7)
#define PCF85263A_FUNCTION_RTCM BIT(4)
#define PCF85263A_FUNCTION_STOPM BIT(3)

// enum {

// }PCF85263A_FUNCTION_PI;

// enum {

// }PCF85263A_FUNCTION_COF;


#define PCF85263A_FLAG_REG 0x2B
#define PCF85263A_FLAG_PIF BIT(7)
#define PCF85263A_FLAG_A2F BIT(6)
#define PCF85263A_FLAG_A1F BIT(5)
#define PCF85263A_FLAG_WDF BIT(4)
#define PCF85263A_FLAG_BSF BIT(3)
#define PCF85263A_FLAG_TSR3F BIT(2)
#define PCF85263A_FLAG_TSR2F BIT(1)
#define PCF85263A_FLAG_TSR1F BIT(0)

#define PCF85263A_RESET_REG 0x2F
#define PCF85263A_RESET_SOFTWARE_RESET 0X2C
#define PCF85263A_RESET_CLEAR_PRESCALER 0XA4
#define PCF85263A_RESET_CLEAR_TIMESTAMP 0X25
#define PCF85263A_RESET_CPR_CTS 		0XA5


#define PCF85263A_STOP_EN_REG 0x2E
#define PCF85263A_STOP BIT(0)


#define PCF85263A_CTRL2 0x01
#define PCF85263A_CTRL2_AIE BIT(7)
#define PCF85263A_CTRL2_AF BIT(6)
#define PCF85263A_CTRL2_MI BIT(5)
#define PCF85263A_CTRL2_HMI BIT(4)
#define PCF85263A_CTRL2_TF BIT(3)

/* CLKOUT frequency selection */
#define PCF85263A_CTRL2_COF_32K 0x0
#define PCF85263A_CTRL2_COF_16K 0x1
#define PCF85263A_CTRL2_COF_8K 0x2
#define PCF85263A_CTRL2_COF_4K 0x3
#define PCF85263A_CTRL2_COF_2K 0x4
#define PCF85263A_CTRL2_COF_1K 0x5
#define PCF85263A_CTRL2_COF_1 0x6
#define PCF85263A_CTRL2_COF_LOW 0x7

#define PCF85263A_OFFSET 0x02
#define PCF85263A_RAM_BYTE 0x03
#define PCF85263A_TENS_SHIFT 4

/* Time related */
#define PCF85263A_SECONDS 0x04
#define PCF85263A_SECONDS_OS BIT(7)

#define PCF85263A_MINUTES 0x05

#define PCF85263A_HOURS 0x06
#define PCF85263A_HOURS_AM_PM BIT(5)

#define PCF85263A_DAYS 0x07

#define PCF85263A_WEEKDAYS 0x08
#define PCF85263A_WEEKDAYS_SUN 0x0
#define PCF85263A_WEEKDAYS_MON 0x1
#define PCF85263A_WEEKDAYS_TUE 0x2
#define PCF85263A_WEEKDAYS_WED 0x3
#define PCF85263A_WEEKDAYS_THU 0x4
#define PCF85263A_WEEKDAYS_FRI 0x5
#define PCF85263A_WEEKDAYS_SAT 0x6

#define PCF85263A_MONTHS 0x09
#define PCF85263A_MONTHS_JAN 0x1
#define PCF85263A_MONTHS_FEB 0x2
#define PCF85263A_MONTHS_MAR 0x3
#define PCF85263A_MONTHS_APR 0x4
#define PCF85263A_MONTHS_MAY 0x5
#define PCF85263A_MONTHS_JUN 0x6
#define PCF85263A_MONTHS_JUL 0x7
#define PCF85263A_MONTHS_AUG 0x8
#define PCF85263A_MONTHS_SEP 0x9
#define PCF85263A_MONTHS_OCT 0x10
#define PCF85263A_MONTHS_NOV 0x11
#define PCF85263A_MONTHS_DEC 0x12

#define PCF85263A_YEARS 0x0a

/* Alarm related */
#define PCF85263A_SECOND_ALARM 0x0b
#define PCF85263A_SECOND_ALARM_EN BIT(7)

#define PCF85263A_MINUTE_ALARM 0x0c
#define PCF85263A_MINUTE_ALARM_EN BIT(7)

#define PCF85263A_HOUR_ALARM 0x0d
#define PCF85263A_HOUR_ALARM_EN BIT(7)
#define PCF85263A_HOUR_ALARM_AM_PM BIT(5)

#define PCF85263A_DAY_ALARM 0x0e
#define PCF85263A_DAY_ALARM_EN BIT_MASK(7)

#define PCF85263A_WEEKDAY_ALARM 0x0f
#define PCF85263A_WEEKDAY_ALARM_EN BIT(7)



struct pcf85263a_time
{
	
	uint8_t millisecond;
	uint8_t second;
	uint8_t minute;
	uint8_t hour;
	uint8_t day;
	uint8_t weekdays;
	uint8_t month;
	uint8_t year;

};


/** @brief Signature for pcf85263a alarm callbacks.
 *
 * The alarm callback is invoked from the system work queue thread.
 * At the point the callback is invoked the corresponding alarm flags
 * will have been cleared from the device status register.  The
 * callback is permitted to invoke operations on the device.
 *
 * @param dev the device from which the callback originated
 * @param id the alarm id
 * @param syncclock the value from maxim_ds3231_read_syncclock() at the
 * time the alarm interrupt was processed.
 * @param user_data 
 */
typedef void (*pcf85263a_alarm_callback_handler_t)(const struct device *dev,
						      uint8_t id,
						      uint32_t syncclock,
						      void *user_data);

int pcf85263a_init(const struct device *dev);
int pcf85263a_osc_en(bool enabled);
int pcf85263a_set_time(struct pcf85263a_time *time);
int pcf85263a_get_time(struct pcf85263a_time *time);
int pcf85263a_set_timer(uint8_t seconds);
int pcf85263a_timer_en(bool enabled);

#endif /* ZEPHYR_DRIVERS_RTC_PCF85263A_PCF85263A_H_ */